// TcpNimPlayer.java

import ch.aplu.jgamegrid.*;
import java.util.*;
import ch.aplu.tcp.*;
import javax.swing.JOptionPane;
import java.awt.Color;

public class TcpNimPlayer extends TcpAgent implements
  GGMouseListener, GGButtonListener
{
  // ------------------- Inner class MyTcpAgentAdapter -------------
  private class MyTcpAgentAdapter implements TcpAgentListener
  {
    private boolean isReady = true;

    public void notifyRelayConnection(boolean connected)
    {
    }

    public void notifyAgentConnection(String agentName, boolean connected)
    {
    }

    public void notifyBridgeConnection(boolean connect)
    {
      if (!connect)
      {
        gg.setTitle("Game server disconnected. Game terminated.");
        gg.setMouseEnabled(false);
        okBtn.setEnabled(false);
      }
    }

    public void dataReceived(String source, int[] data)
    {
      if (!isReady)
        return;

      String str = "";
      switch (data[0])
      {
        case Command.GAME_RUNNING:
          gg.setTitle("Game in process.\nPlease wait for next game.\n"
            + "Terminating now...");
          TcpTools.delay(5000);
          System.exit(0);
          break;

        case Command.PLAYER_DISCONNECT:
          disconnectedPlayer = TcpTools.intAryToString(data, 1);
          break;

        case Command.NUMBER_OF_PLAYERS:
          initPlayground();
          if (!disconnectedPlayer.equals(""))
            disconnectedPlayer = "Disconnected: " + disconnectedPlayer + ". ";
          gg.setTitle(disconnectedPlayer
            + "Current number in group: " + data[1]
            + ". Waiting for more players to join...");
          disconnectedPlayer = "";
          break;

        case Command.GAME_STARTING:
          str = "Group complete. In group: ";
          String[] names =
            TcpTools.split(TcpTools.intAryToString(data, 1), "&&");
          for (int i = 0; i < names.length; i++)
          {
            if (i < names.length - 1)
              str += names[i] + "; ";
            else
              str += names[i];
          }
          gg.setTitle(str + ". Wait start selection...");
          break;

        case Command.INIT_PLAYGROUND:
          nbMatches = data[1];
          initPlayground();
          break;

        case Command.REPORT_POSITION:
          Location loc = new Location(data[1], 4);
          gg.getOneActorAt(loc).removeSelf();
          nbMatches--;
          gg.setTitle(nbMatches + " left. " + remotePlayer + " is playing.");
          gg.refresh();
          break;

        case Command.LOCAL_MOVE:
          gg.activate();
          gg.setTitle(nbMatches + " left. It's you to play.");
          nbTaken = 0;
          gg.setMouseEnabled(true);
          okBtn.setEnabled(true);
          break;

        case Command.REMOTE_MOVE:
          remotePlayer = TcpTools.intAryToString(data, 1);
          gg.setTitle(nbMatches + " left. " + remotePlayer + " will play.");
          break;

        case Command.WINNER:
          String looser = TcpTools.intAryToString(data, 1);
          gg.setTitle("Game over. Player " + looser + " lost the game.");
          break;

        case Command.LOOSER:
          gg.setTitle("Game over. You lost the game.");
          break;
      }
    }
  }

  // --------------- Inner class Match --------------
  private class Match extends Actor
  {
    public Match()
    {
      super("sprites/match.gif");
    }
  }
  // ------------------- End of inner class ------------------------
  //
  private final static String serverName = "NimServer";
  private final static String sessionID = "apx%3z";
  private String agentName;
  private String remotePlayer = "";
  private String disconnectedPlayer = "";
  private int nbMatches;
  private GameGrid gg;
  private GGBackground bg;
  private GGButton okBtn = new GGButton("sprites/ok.gif", true);
  private int nbTaken;

  // Protocol tags
  private interface Command
  {
    int GAME_RUNNING = 0;
    int NUMBER_OF_PLAYERS = 1;
    int INIT_PLAYGROUND = 2;
    int GAME_STARTING = 3;
    int REMOTE_MOVE = 4;
    int LOCAL_MOVE = 5;
    int REPORT_OK = 6;
    int REPORT_POSITION = 7;
    int WINNER = 8;
    int LOOSER = 9;
    int PLAYER_DISCONNECT = 10;
  }

  public TcpNimPlayer()
  {
    super(sessionID, serverName);
    agentName = requestEntry("Enter your name (ASCII >3 chars):");
    initGameWindow();
    gg.addMouseListener(this, GGMouse.lPress);
    gg.setTitle("Connecting to relay " + getRelay() + "...");
    addTcpAgentListener(new MyTcpAgentAdapter());
    ArrayList<String> connectList =
      connectToRelay(agentName, 6000);
    if (connectList.isEmpty())
    {
      gg.setTitle("Connection to relay failed. Terminating now...");
      TcpTools.delay(3000);
      System.exit(1);
    }
    gg.setTitle("Connection established. Personal name: " + connectList.get(0));
    // Game server must be up and running
    if (!isBridgeConnected())
    {
      gg.setTitle("Game server not found. Terminating now...");
      TcpTools.delay(3000);
      System.exit(1);
    }
  }

  private void initGameWindow()
  {
    gg = new GameGrid(56, 9, 12, false);
    bg = gg.getBg();
    bg.clear(Color.blue);
    gg.addActor(okBtn, new Location(50, 4));
    okBtn.addButtonListener(this);
    gg.addMouseListener(this, GGMouse.lPress);
    gg.show();
  }

  private void initPlayground()
  {
    okBtn.setEnabled(false);
    gg.setMouseEnabled(false);
    gg.removeActors(Match.class);
    for (int i = 0; i < nbMatches; i++)
      gg.addActor(new Match(), new Location(2 + 3 * i, 4));
  }

  public boolean mouseEvent(GGMouse mouse)
  {
    Location loc = gg.toLocationInGrid(mouse.getX(), mouse.getY());
    Actor actor = null;
    for (int y = 2; y < 7; y++)
    {
      actor = gg.getOneActorAt(new Location(loc.x, y), Match.class);
      if (actor != null)
        break;
    }
    if (actor != null)
    {
      if (nbTaken == 3)
        gg.setTitle("Take a maximum of 3. Click 'OK' to continue");
      else
      {
        actor.removeSelf();
        sendCommand(agentName, Command.REPORT_POSITION, actor.getLocation().x);
        nbMatches--;
        gg.setTitle(nbMatches + " matches remaining. Click 'OK' to continue");
        nbTaken++;
        gg.refresh();
      }
    }
    return false;
  }

  public void buttonClicked(GGButton button)
  {
    if (nbTaken == 0)
      gg.setTitle("You have to remove at least 1 match");
    else
    {
      sendCommand(agentName, Command.REPORT_OK);
      okBtn.setEnabled(false);
      gg.setMouseEnabled(false);
    }
  }

  public void buttonPressed(GGButton button)
  {
  }

  public void buttonReleased(GGButton button)
  {
  }

  private String requestEntry(String prompt)
  {
    String entry = "";
    while (entry.length() < 1)
    {
      entry = JOptionPane.showInputDialog(null, prompt, "");
      if (entry == null)
        System.exit(0);
    }
    return entry.trim();
  }

  public static void main(String[] args)
  {
    new TcpNimPlayer();
  }
}
